@@ -23,7 +23,6 @@
 #include <diskio/chunkmanager.h>
 #include <peer/peer.h>
 #include <peer/peermanager.h>
-#include <peer/packetwriter.h>
 #include <krandom.h>
 
 
@@ -50,7 +49,7 @@ namespace bt
 	bool AdvancedChokeAlgorithm::calcACAScore(Peer* p,ChunkManager & cman,const TorrentStats & stats)
 	{
 		const PeerInterface::Stats & s = p->getStats();
-		if (p->isSeeder())
+		if (p->isSeeder() || s.partial_seed)
 		{
 			/*
 			double bd = 0;
@@ -155,12 +154,12 @@ namespace bt
 			Peer* p = ppl.at(i);
 			if (!poup && num_unchoked < num_slots)
 			{
-				p->getPacketWriter().sendUnchoke();
+				p->sendUnchoke();
 				num_unchoked++;
 			}
 			else if (num_unchoked < num_slots -1 || p == poup)
 			{
-				p->getPacketWriter().sendUnchoke();
+				p->sendUnchoke();
 				if (p != poup)
 					num_unchoked++;
 			}
